#include <iostream>
#include <fstream>

#include <list_graph.hh>
#include <dimacs.hh>
#include <edmonds_karp.hh>
#include <time_measure.h>

using namespace hugo;

// Use a DIMACS max flow file as stdin.
// read_dimacs_demo < dimacs_max_flow_file
int main(int, char **) {
  typedef ListGraph::NodeIt NodeIt;
  typedef ListGraph::EachEdgeIt EachEdgeIt;

  ListGraph G;
  NodeIt s, t;
  ListGraph::EdgeMap<int> cap(G);
  readDimacsMaxFlow(std::cin, G, s, t, cap);

  {
  std::cout << "edmonds karp demo (blocking flow augmentation)..." << std::endl;
  ListGraph::EdgeMap<int> flow(G); //0 flow

  double pre_time=currTime();
  MaxFlow<ListGraph, int, ListGraph::EdgeMap<int>, ListGraph::EdgeMap<int> > max_flow_test(G, s, t, flow, cap);
  //max_flow_test.augmentWithBlockingFlow<ListGraph>();
  max_flow_test.run<ListGraph>();
  double post_time=currTime();

  //std::cout << "maximum flow: "<< std::endl;
  //for(EachEdgeIt e=G.first<EachEdgeIt>(); e.valid(); ++e) { 
  //  std::cout<<"("<<G.tail(e)<< "-"<<flow.get(e)<<"->"<<G.head(e)<<") ";
  //}
  //std::cout<<std::endl;
  std::cout << "elapsed time: " << post_time-pre_time << " sec"<< std::endl; 
  std::cout << "flow value: "<< max_flow_test.flowValue() << std::endl;
  }

  {
  std::cout << "edmonds karp demo (shortest path augmentation)..." << std::endl;
  ListGraph::EdgeMap<int> flow(G); //0 flow

  double pre_time=currTime();
  MaxFlow<ListGraph, int, ListGraph::EdgeMap<int>, ListGraph::EdgeMap<int> > max_flow_test(G, s, t, flow, cap);
  //max_flow_test.augmentWithBlockingFlow<ListGraph>();
  max_flow_test.run();
  double post_time=currTime();

  //std::cout << "maximum flow: "<< std::endl;
  //for(EachEdgeIt e=G.first<EachEdgeIt>(); e.valid(); ++e) { 
  //  std::cout<<"("<<G.tail(e)<< "-"<<flow.get(e)<<"->"<<G.head(e)<<") ";
  //}
  //std::cout<<std::endl;
  std::cout << "elapsed time: " << post_time-pre_time << " sec"<< std::endl; 
  std::cout << "flow value: "<< max_flow_test.flowValue() << std::endl;
  }

  return 0;
}
